#include "anime.h"
#include "libmy/lex.h"
#include "libmy/key.h"
#include "res.h"
#include "img.h"
#include "bgm.h"
#include "se.h"
#include "nv.h"


//---------------------------------------------------------------------------
ST_ANIME_TABLE AnimePat[ANIME_MAX_PAT_CNT] = {
	{ "img1",     (void*)AnimeExecImg1    },
	{ "img2",     (void*)AnimeExecImg2    },
	{ "img3",     (void*)AnimeExecImg3    },
	{ "img4",     (void*)AnimeExecImg4    },
	{ "img5",     (void*)AnimeExecImg5    },
	{ "img_val",  (void*)AnimeExecImgVal  },
	{ "img_line", (void*)AnimeExecImgLine },
	{ "effect1",  (void*)AnimeExecEffect1 },
	{ "effect2",  (void*)AnimeExecEffect2 },
	{ "fill",     (void*)AnimeExecFill    },
	{ "wait",     (void*)AnimeExecWait    },
	{ "key_wait", (void*)AnimeExecKeyWait },
	{ "bgm",      (void*)AnimeExecBgm     },
	{ "se",       (void*)AnimeExecSe      },
	{ "loop",     (void*)AnimeExecLoop    },
	{ "inc",      (void*)AnimeExecInc     },
	{ "dec",      (void*)AnimeExecDec     },
	{ "jae" ,     (void*)AnimeExecJae     },
	{ "jbe" ,     (void*)AnimeExecJbe     },
	{ "end",      (void*)AnimeExecEnd     },
};

char* AnimeDat[ANIME_MAX_DAT_CNT] = {
	(char*)&ani_1_op_txt,
	(char*)&ani_2_ed_txt,
	(char*)&ani_3_hs_txt,
	(char*)&ani_4_pa_txt,
};

//---------------------------------------------------------------------------
ST_ANIME Anime;


//---------------------------------------------------------------------------
EWRAM_CODE void AnimeInit(void)
{
	_Memset(&Anime, 0x00, sizeof(ST_ANIME));

	Anime.act = ANIME_ACT_END;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeSetDat(u32 no)
{
	_ASSERT(no < ANIME_MAX_DAT_CNT);
	TRACEOUT("[AnimeSetDat: no:0x%x]\n", no);

	AnimeInit();
	LexInit();
	LexSetCur(AnimeDat[no]);

	Anime.act = ANIME_ACT_EXEC;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExec(void)
{
	if(AnimeIsEnd() == TRUE || ImgIsEnd() == FALSE)
	{
		return;
	}

	if(Anime.waitCnt != 0)
	{
		Anime.waitCnt--;
		return;
	}

	if(Anime.isKey == TRUE && KeyIsTrgAB() == FALSE)
	{
		return;
	}
	Anime.isKey = FALSE;


	Anime.isLoop = TRUE;

	do
	{
		AnimeExecSub();

	} while(Anime.isLoop == TRUE);
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecSub(void)
{
	char* p = LexGetStr();
	TRACEOUT("[Anime %s]\n", p);

	u32 i;

	for(i=0; i<ANIME_MAX_PAT_CNT; i++)
	{
		if(_Strncmp(p, AnimePat[i].pStr, ANIME_MAX_STR_SIZE) == 0)
		{
			break;
		}
	}
	_ASSERT(i < ANIME_MAX_PAT_CNT);


	AnimePat[i].pFunc();
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecImg1(void)
{
	u32 num = LexGetNum();
	ImgSetBg(num);

	Anime.isLoop = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecImg2(void)
{
	u32 num = LexGetNum();
	u32 x   = LexGetNum();
	u32 y   = LexGetNum();

	ImgDirectCrop(num, x, y);

	Anime.isLoop = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecImg3(void)
{
	u32 num = LexGetNum();
	u32 x   = LexGetNum();
	u32 y   = LexGetNum();

	ImgDirectBlend(num, x, y);

	Anime.isLoop = FALSE;
}
//---------------------------------------------------------------------------
// gV[pGtFNgP
EWRAM_CODE void AnimeExecImg4(void)
{
	u32 num = LexGetNum();
	u32 x   = LexGetNum();
	u32 y   = LexGetNum();
	u32 gx  = Anime.var;
	u32 gy  = LexGetNum();
	u32 gs  = LexGetNum();

	ImgDirectClip(num, x, y, gx, gy, gs);

	Anime.isLoop = FALSE;
}
//---------------------------------------------------------------------------
// gV[pGtFNgQ
EWRAM_CODE void AnimeExecImg5(void)
{
	u32 num = LexGetNum();
	u32 x   = LexGetNum();
	u32 y   = LexGetNum();
	u32 gx  = LexGetNum();
	u32 gy  = Anime.var;
	u32 gs  = LexGetNum();

	ImgDirectClip(num, x, y, gx, gy, gs);

	Anime.isLoop = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecImgVal(void)
{
	u32 num = LexGetNum();
	u16 val = NvGetVar(num);

	ImgSetBg(val);

	Anime.isLoop = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecImgLine(void)
{
	u32 num = LexGetNum();
	u16 cnt = Anime.var;

	if(cnt >= 160)
	{
		cnt = 159;
	}
	ImgDirectLine(num, cnt);

	Anime.isLoop = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecEffect1(void)
{
	u32 num  = LexGetNum();
	u32 wait = LexGetNum();

	ImgSetEffect(num, wait);

	Anime.isLoop = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecEffect2(void)
{
	u32 num  = LexGetNum();
	u32 wait = LexGetNum();

	ImgSetEffectOnly(num, wait);

	Anime.isLoop = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecFill(void)
{
	u32 r = LexGetNum();
	u32 g = LexGetNum();
	u32 b = LexGetNum();

	ImgDirectFill(RGB5(r, g, b));

	Anime.isLoop = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecWait(void)
{
	Anime.waitCnt = LexGetNum();
	Anime.isLoop  = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecKeyWait(void)
{
	Anime.isKey  = TRUE;
	Anime.isLoop = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecBgm(void)
{
	u32 num  = LexGetNum();
	u32 num2 = LexGetNum();

	bool isLoop = (num2 == 1) ? TRUE : FALSE;
	BgmPlay(num, isLoop);

	Anime.isLoop  = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecSe(void)
{
	u32 num  = LexGetNum();
	u32 num2 = LexGetNum();

	bool isLoop = (num2 == 1) ? TRUE : FALSE;
	SePlay(num, isLoop);

	Anime.isLoop  = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecLoop(void)
{
	Anime.var  = LexGetNum();
	Anime.pCur = LexGetCur();
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecInc(void)
{
	Anime.var++;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecDec(void)
{
	Anime.var--;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecJae(void)
{
	s32 v = LexGetNum();

	if(Anime.var >= v)
	{
		LexSetCur(Anime.pCur);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecJbe(void)
{
	s32 v = LexGetNum();

	if(Anime.var <= v)
	{
		LexSetCur(Anime.pCur);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecEnd(void)
{
	Anime.act = ANIME_ACT_END;
	Anime.isLoop = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool AnimeIsEnd(void)
{
	return (Anime.act == ANIME_ACT_END) ? TRUE : FALSE;
}
