#include "menu.h"
#include "libmy/key.h"
#include "text.h"
#include "siori.h"
#include "nv.h"
#include "history.h"
#include "img.h"
#include "bgm.h"


//---------------------------------------------------------------------------
// bf[h@wiԍ̃e[u
const u16 const MenuCgTable[MENU_MAX_CG_CNT] = {
	 51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62, 100, 101, 102, 103, 107, 113, 114, 117,
	118, 119, 121, 122, 123, 127, 129, 130, 133, 134, 135, 136, 137, 139, 140, 141, 156, 200, 201, 203,
	206, 209, 211, 212, 213, 218, 219, 221, 222, 223, 227, 228, 230, 231, 232, 233, 245, 246, 250, 253,
	255, 256, 260, 300, 301, 302, 303, 309, 310, 312, 313, 315, 318, 319, 320, 321, 322, 327, 328, 329,
	330, 331, 332, 333, 334, 348, 349, 350, 351, 352, 353, 400, 401, 402, 405, 408, 409, 410, 411, 413,
	414, 415, 416, 417, 418, 419, 420, 423, 424, 425, 433, 434, 435, 436, 438, 442, 500, 502, 504, 508,
	509, 512, 516, 517, 524, 526, 527, 528, 529, 530, 531, 546, 547, 548, 589, 590, 592, 601, 604, 608,
	613, 617, 618, 619, 624, 625, 626, 627, 629, 630, 631, 636, 637, 638, 639, 640, 641, 642, 646, 647,
	649, 650, 651, 652, 653, 654, 655, 657, 658, 659, 660, 661, 662, 695, 696, 732, 750, 751
};

ST_MENU Menu;


//---------------------------------------------------------------------------
EWRAM_CODE void MenuInit(void)
{
	_Memset(&Menu, 0x00, sizeof(ST_MENU));

	MenuSetAct(MENU_ACT_END);
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuExec(void)
{
	if(MenuIsEnd() == TRUE)
	{
		return;
	}


	switch(Menu.act)
	{
	case MENU_ACT_READY:
		TextClearDat();
		MenuSetAct(MENU_ACT_DRAW);
		break;

	case MENU_ACT_DRAW:
		MenuExecDraw();
		break;

	case MENU_ACT_SELECT:
		MenuExecSelect();
		break;

	case MENU_ACT_SELECTED:
		MenuExecSelected();
		break;

	case MENU_ACT_END_READY:
		TextDrawRestore();
		MenuSetAct(MENU_ACT_END);
		break;

	default:
		SystemError("[Err] MenuExec Menu.act=%d\n", Menu.act);
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuExecDraw(void)
{
	TextDrawStrXy(Menu.sx, Menu.sy, (u16*)Menu.title);

	switch(Menu.type)
	{
	case MENU_TYPE_NORMAL:
		MenuExecDrawNormal();
		break;

	case MENU_TYPE_OPTION:
		MenuExecDrawOption();
		break;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuExecDrawNormal(void)
{
	s32 i;

	for(i=0; i<Menu.regCnt; i++)
	{
		if(i == Menu.sel)
		{
			TextDrawStrXy(Menu.sx, Menu.sy + 2 + i, (u16*)Menu.item[i].str);
		}
		else
		{
			TextDrawStrXyGray(Menu.sx, Menu.sy + 2 + i, (u16*)Menu.item[i].str);
		}
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuExecDrawOption(void)
{
	MenuExecDrawNormal();

	s32 i;

	for(i=0; i<Menu.regCnt; i++)
	{
		u32 cnt = Menu.item[i].cnt;

		// 0x82 0x4f = SJISR[huOv
		char str[5] ALIGN(4);
		str[0] = 0x82;
		str[1] = 0x4f + Mod(Div(cnt, 10), 10);
		str[2] = 0x82;
		str[3] = 0x4f + Mod(cnt, 10);
		str[4] = '\0';

		if(i == Menu.sel)
		{
			TextDrawStrXy(Menu.sx + 7, Menu.sy + 2 + i, (u16*)str);
		}
		else
		{
			TextDrawStrXyGray(Menu.sx + 7, Menu.sy + 2 + i, (u16*)str);
		}
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuExecSelect(void)
{
	ST_MENU* p = &Menu;

	p->trg = KeyGetTrg();

	if((p->trg & KEY_B) && (p->isCancel == TRUE))
	{
		if(p->ret == MENU_RET_TITLE)
		{
			MenuSetTitle();
		}
		else if(p->ret == MENU_RET_SYSTEM)
		{
			MenuSetSystem();
		}
		else
		{
			MenuSetAct(MENU_ACT_END_READY);
		}
	}
	else if((p->trg & KEY_UP) && p->sel > 0)
	{
		p->sel--;
		MenuSetAct(MENU_ACT_DRAW);
	}
	else if((p->trg & KEY_DOWN) && (p->sel + 1) < p->regCnt)
	{
		p->sel++;
		MenuSetAct(MENU_ACT_DRAW);
	}
	else if((p->trg & KEY_LEFT) || (p->trg & KEY_RIGHT) || (p->trg & KEY_A))
	{
		if(p->sel != -1)
		{
			MenuSetAct(MENU_ACT_SELECTED);
		}
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuExecSelected(void)
{
	Menu.item[Menu.sel].pFunc();
}
//---------------------------------------------------------------------------
EWRAM_CODE bool MenuIsEnd(void)
{
	return (Menu.act == MENU_ACT_END) ? TRUE : FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuSetAct(u32 act)
{
	Menu.act = act;
	Menu.actStep = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuSetOption(u32 sx, u32 sy, char* pTitle, u32 type, bool isCancel)
{
	Menu.sx = sx;
	Menu.sy = sy;
	Menu.sel = 0;
	Menu.ret = MENU_RET_NORMAL;
	Menu.type = type;
	Menu.regCnt = 0;
	Menu.isCancel = isCancel;
	_Strncpy(Menu.title, pTitle, MENU_MAX_STR_LEN);

	MenuSetAct(MENU_ACT_READY);
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuAddItem(char* pStr, void* pFunc)
{
	_ASSERT(Menu.regCnt <  MENU_MAX_ITEM_CNT);

	_Strncpy(Menu.item[Menu.regCnt].str, pStr, MENU_MAX_STR_LEN);
	Menu.item[Menu.regCnt].pFunc = pFunc;
	Menu.item[Menu.regCnt].cnt = 0;

	Menu.regCnt++;
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuSetTitle(void)
{
	MenuInit();
	MenuSetOption(0, 0, "fA{J@ch`ankhptd", MENU_TYPE_NORMAL, FALSE);

	MenuAddItem("͑I",   &MenuItem0Chapter);
	MenuAddItem("[h",   &MenuItem0Load);
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuSetChapter(void)
{
	MenuInit();
	MenuSetOption(5, 0, "|@͑I@|", MENU_TYPE_NORMAL, TRUE);

	MenuAddItem("I[vjO", &MenuItem1Chapter);
	MenuAddItem("E",     &MenuItem1Chapter);
	MenuAddItem("y",       &MenuItem1Chapter);
	MenuAddItem("i",         &MenuItem1Chapter);
	MenuAddItem("",           &MenuItem1Chapter);
	MenuAddItem("V@O",   &MenuItem1Chapter);
	MenuAddItem("V@",   &MenuItem1Chapter);
	MenuAddItem("V@",   &MenuItem1Chapter);
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuSetSystem(void)
{
	MenuInit();
	MenuSetOption(3, 0, "|@VXej[@|", MENU_TYPE_NORMAL, TRUE);

	MenuAddItem("",     &MenuItem2Next);
	MenuAddItem("",   &MenuItem2TextHide);
	MenuAddItem("ViIz", &MenuItem2Ref);
	MenuAddItem("Z[u",       &MenuItem2Save);
	MenuAddItem("[h",       &MenuItem2Load);
	MenuAddItem("IvV",   &MenuItem2Option);
	MenuAddItem("Q[I",   &MenuItem2GameEnd);
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuSetSystemOption(void)
{
	MenuInit();
	MenuSetOption(3, 0, "|@VXeݒ@|", MENU_TYPE_OPTION, TRUE);

	MenuAddItem("EFCg", &MenuItem4TextWait);
	MenuAddItem("wiPx@@", &MenuItem4Fade);
	MenuAddItem("afl@@@", &MenuItem4Bgm);
	MenuAddItem("bf@@@@", &MenuItem4Cg);

	Menu.item[0].cnt = TextGetWaitNum();
	Menu.item[1].cnt = ImgGetFade();
	Menu.item[2].cnt = 2;
	Menu.item[3].cnt = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuSetSave(void)
{
	MenuInit();
	MenuSetOption(3, 0, "@@|@Z[u@|", MENU_TYPE_NORMAL, TRUE);

	u32 i;

	for(i=0; i<SIORI_MAX_CNT; i++)
	{
		MenuAddItem(SioriGetName(i), &MenuItem3SaveSiori);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuSetLoad(void)
{
	MenuInit();
	MenuSetOption(3, 0, "@@|@[h@|", MENU_TYPE_NORMAL, TRUE);

	u32 i;

	for(i=0; i<SIORI_MAX_CNT; i++)
	{
		MenuAddItem(SioriGetName(i), &MenuItem3LoadSiori);
	}
}
//---------------------------------------------------------------------------
// ͑I
EWRAM_CODE void MenuItem0Chapter(void)
{
	if(Menu.trg & KEY_A)
	{
		MenuSetChapter();
		Menu.ret = MENU_RET_TITLE;

		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// [h
EWRAM_CODE void MenuItem0Load(void)
{
	if(Menu.trg & KEY_A)
	{
		MenuSetLoad();
		Menu.ret = MENU_RET_TITLE;

		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// ͂I
EWRAM_CODE void MenuItem1Chapter(void)
{
	if(Menu.trg & KEY_A)
	{
		NvSetCapter(Menu.sel);
		MenuSetAct(MENU_ACT_END_READY);

		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// 
EWRAM_CODE void MenuItem2Next(void)
{
	if(Menu.trg & KEY_A)
	{
		NvSetSkip(TRUE);
		MenuSetAct(MENU_ACT_END_READY);

		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// 
EWRAM_CODE void MenuItem2TextHide(void)
{
	switch(Menu.actStep)
	{
	case 0:
		if(Menu.trg & KEY_A)
		{
			Menu.actStep++;
		}
		else
		{
			MenuSetAct(MENU_ACT_SELECT);
		}
		break;

	case 1:
		TextClearDat();
		Menu.actStep++;
		break;

	case 2:
		Menu.trg = KeyGetTrg();

		if(Menu.trg & KEY_A)
		{
			MenuSetAct(MENU_ACT_DRAW);
		}
	}
}
//---------------------------------------------------------------------------
// ViIz
EWRAM_CODE void MenuItem2Ref(void)
{
	switch(Menu.actStep)
	{
	case 0:
		if(Menu.trg & KEY_A && HistoryIsEmpty() == FALSE)
		{
			Menu.actStep++;
		}
		else
		{
			MenuSetAct(MENU_ACT_SELECT);
		}
		break;

	case 1:
		HistoryInitAct(FALSE);
		Menu.actStep++;
		break;

	case 2:
		HistoryExec();

		if(HistoryIsEnd() == TRUE)
		{
			MenuSetAct(MENU_ACT_READY);
		}
		break;
	}
}
//---------------------------------------------------------------------------
// Z[u@VXej[
EWRAM_CODE void MenuItem2Save(void)
{
	if(Menu.trg & KEY_A)
	{
		MenuSetSave();
		Menu.ret = MENU_RET_SYSTEM;

		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// [h@VXej[
EWRAM_CODE void MenuItem2Load(void)
{
	if(Menu.trg & KEY_A)
	{
		MenuSetLoad();
		Menu.ret = MENU_RET_SYSTEM;

		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// IvV
EWRAM_CODE void MenuItem2Option(void)
{
	if(Menu.trg & KEY_A)
	{
		MenuSetSystemOption();
		Menu.ret = MENU_RET_SYSTEM;

		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// Q[I
EWRAM_CODE void MenuItem2GameEnd(void)
{
	if(Menu.trg & KEY_A)
	{
		HistoryInit();
		TextInit();

		NvSetAct(NV_ACT_END_READY);
		MenuSetAct(MENU_ACT_END_READY);
		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// Z[u@PIς
EWRAM_CODE void MenuItem3SaveSiori(void)
{
	if(Menu.trg & KEY_A)
	{
		SioriSave(Menu.sel);
		MenuSetAct(MENU_ACT_END_READY);

		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// [h@PIς
EWRAM_CODE void MenuItem3LoadSiori(void)
{
	if(Menu.trg & KEY_A)
	{
		if(SioriLoad(Menu.sel) == TRUE)
		{
			MenuSetAct(MENU_ACT_END_READY);
			return;
		}
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// EFCg
EWRAM_CODE void MenuItem4TextWait(void)
{
	if(Menu.trg & KEY_LEFT && Menu.item[0].cnt > 0)
	{
		Menu.item[0].cnt--;
		TextSetWaitNum(Menu.item[0].cnt);

		MenuSetAct(MENU_ACT_DRAW);
		return;
	}
	else if(Menu.trg & KEY_RIGHT && Menu.item[0].cnt < 10)
	{
		Menu.item[0].cnt++;
		TextSetWaitNum(Menu.item[0].cnt);

		MenuSetAct(MENU_ACT_DRAW);
		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// wiPx
EWRAM_CODE void MenuItem4Fade(void)
{
	if(Menu.trg & KEY_LEFT && Menu.item[1].cnt > 0)
	{
		Menu.item[1].cnt--;
		ImgSetFade(Menu.item[1].cnt);

		MenuSetAct(MENU_ACT_DRAW);
		return;
	}
	else if(Menu.trg & KEY_RIGHT && Menu.item[1].cnt < 16)
	{
		Menu.item[1].cnt++;
		ImgSetFade(Menu.item[1].cnt);

		MenuSetAct(MENU_ACT_DRAW);
		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// afl
EWRAM_CODE void MenuItem4Bgm(void)
{
	if(Menu.trg & KEY_LEFT && Menu.item[2].cnt > 2)
	{
		Menu.item[2].cnt--;
		MenuSetAct(MENU_ACT_DRAW);
		return;
	}
	else if(Menu.trg & KEY_RIGHT && Menu.item[2].cnt < 24)
	{
		Menu.item[2].cnt++;
		MenuSetAct(MENU_ACT_DRAW);
		return;
	}
	else if(Menu.trg & KEY_A)
	{
		BgmPlay2(Menu.item[2].cnt, TRUE);
		MenuSetAct(MENU_ACT_DRAW);
		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
//---------------------------------------------------------------------------
// bf
EWRAM_CODE void MenuItem4Cg(void)
{
	if(Menu.trg & KEY_LEFT && Menu.item[3].cnt > 0)
	{
		Menu.item[3].cnt--;
		MenuSetAct(MENU_ACT_DRAW);
		return;
	}
	else if(Menu.trg & KEY_RIGHT && Menu.item[3].cnt < MENU_MAX_CG_CNT-1)
	{
		Menu.item[3].cnt++;
		MenuSetAct(MENU_ACT_DRAW);
		return;
	}
	else if(Menu.trg & KEY_A)
	{
		ImgDirectCrop(MenuCgTable[Menu.item[3].cnt], 0, 0);
		ImgSetEffectOnly(IMG_EFFECT_NORMAL, 0);

		MenuSetAct(MENU_ACT_DRAW);
		return;
	}

	MenuSetAct(MENU_ACT_SELECT);
}
