#include "nv.h"
#include "nv2.h"
#include "libmy/key.h"
#include "file.h"
#include "history.h"
#include "img.h"
#include "anime.h"
#include "menu.h"
#include "bgm.h"
#include "se.h"


//---------------------------------------------------------------------------
ST_NV Nv;


//---------------------------------------------------------------------------
EWRAM_CODE void NvInit(void)
{
	_Memset(&Nv, 0x00, sizeof(ST_NV));
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExec(void)
{
	if(ImgIsEnd() == FALSE || TextIsEnd() == FALSE || MenuIsEnd() == FALSE || AnimeIsEnd() == FALSE || HistoryIsEnd() == FALSE)
	{
		return;
	}

	if(NvIsEnd() == TRUE)
	{
		return;
	}


	switch(Nv.act)
	{
	case NV_ACT_PARSE:
		NvExecParse();
		break;

	case NV_ACT_KEY:
		NvExecKey();
		break;

	case NV_ACT_SELECT:
		NvExecSelect();
		break;

	case NV_ACT_END_READY:
		NvExecEndReady();
		break;

	default:
		SystemError("[Err] NvExec Nv.act = %d\n", Nv.act);
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecKey(void)
{
	if(NvExecKeySub() == TRUE)
	{
		TextSetLf();
		NvSetAct(NV_ACT_PARSE);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE bool NvExecKeySub(void)
{
	u16 cnt = KeyGetCnt();
	u16 trg = KeyGetTrg();

	TextShowCur();

	if(Nv.isSkip == TRUE)
	{
		if(cnt & KEY_B)
		{
			Nv.isSkip = FALSE;
		}
		else
		{
			TextHideCur();
			return TRUE;
		}
	}

	if((cnt & KEY_R) || (cnt & KEY_SELECT) || (trg & KEY_A) || (trg & KEY_DOWN))
	{
		TextHideCur();
		return TRUE;
	}
	else if((trg & KEY_LEFT) && HistoryIsEmpty() == FALSE)
	{
		TextHideCur();
		HistoryInitAct(TRUE);

		return FALSE;
	}
	else if(trg & KEY_B)
	{
		TextHideCur();
		MenuSetSystem();

		return FALSE;
	}

	return FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecSelect(void)
{
	s32 i;

	switch(Nv.actStep)
	{
	case 0:
		TextClear();
		TextShow();
		Nv.sel = -1;

		TRACEOUT("[%x]\n", &i);
		Nv.actStep++;
		break;

	case 1:
		for(i=0; i<Nv.selCnt; i++)
		{
			if(Nv.sel == i)
			{
				TextDrawStrXy(0, i, Nv.selBuf[i]);
			}
			else
			{
				TextDrawStrXyGray(0, i, Nv.selBuf[i]);
			}
		}
		Nv.actStep++;
		break;

	case 2: ;
		u16 trg = KeyGetTrg();

		if((trg & KEY_UP) && Nv.sel > 0)
		{
			Nv.sel--;
			Nv.actStep--;
		}
		else if((trg & KEY_DOWN) && (Nv.sel + 1) < Nv.selCnt)
		{
			Nv.sel++;
			Nv.actStep--;
		}
		else if(((trg & KEY_A)  || (trg & KEY_RIGHT)) && Nv.sel != -1)
		{
			Nv.actStep++;
		}
		else if((trg & KEY_LEFT) && HistoryIsEmpty() == FALSE)
		{
			HistoryInitAct(TRUE);
			Nv.actStep--;
		}
		else if(trg & KEY_B)
		{
			MenuSetSystem();
			Nv.actStep--;
		}
		break;

	case 3:
		TextClearDat();
		TextClearXY();

		Nv.pCur   = Nv.pHead + Nv.selAdr[Nv.sel];
		Nv.selCnt = 0;
		Nv.isSkip = FALSE;

		NvSetAct(NV_ACT_PARSE);
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecEndReady(void)
{
	BgmStop();
	SeStop();

	TextClear();
	TextShow();

	NvSetAct(NV_ACT_END);
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvSetAct(u32 act)
{
	Nv.act = act;
	Nv.actStep = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvSetSco(u32 no, u32 adr)
{
	ST_NV_SCO_HEADER* p = (ST_NV_SCO_HEADER*)FileGetSco(no);

	Nv.datAdr = p->datAdr;
	Nv.size   = p->size;
	Nv.no     = no;
	Nv.pHead  = (u8*)p;
	Nv.pCur   = (u8*)p + p->datAdr;

	if(adr != 0)
	{
		Nv.pCur = (u8*)p + adr;
	}

	TRACEOUT("[Load:%s no:%d pCur:0x%x]\n", FileGetScoStr(Nv.no), Nv.no, Nv.pCur - Nv.pHead);
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvSetSkip(bool is)
{
	Nv.isSkip = is;
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvSetCapter(u32 no)
{
	NvInit();

	switch(no)
	{
	// I[vjO
	case 0:
		AnimeSetDat(0);
		// no break;

	// A E
	case 1:
		NvSetSco(22, 0);
		break;

	// B y
	case 2:
		NvSetSco(51, 0);
		break;

	// C i
	case 3:
		NvSetSco(78, 0);
		break;

	// D 
	case 4:
		NvSetSco(107, 0);
		break;

	// E V@O
	case 5:
		NvSetSco(126, 0);
		break;

	// F V@
	case 6:
		NvSetSco(150, 0);
		break;

	// G V@
	case 7:
		NvSetSco(170, 0);
		break;
	}

	Nv.act = NV_ACT_PARSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvSetVar(u16 num, u16 val)
{
	_ASSERT(num < NV_MAX_VAR_CNT);
	TRACEOUT("[var%d Set %d]\n", num, val);

	Nv.var[num] = val;
}
//---------------------------------------------------------------------------
EWRAM_CODE u16 NvGetVar(u16 num)
{
	_ASSERT(num < NV_MAX_VAR_CNT);
	TRACEOUT("[var%d Get %d]\n", num, Nv.var[num]);

	return Nv.var[num];
}
//---------------------------------------------------------------------------
EWRAM_CODE bool NvIsEnd(void)
{
	return (Nv.act == NV_ACT_END) ? TRUE : FALSE;
}
