#include "nv2.h"
#include "nv.h"
#include "libmy/mt.h"
#include "anime.h"
#include "img.h"
#include "text.h"
#include "bgm.h"
#include "se.h"
#include "file.h"

//---------------------------------------------------------------------------
extern ST_NV Nv;


//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParse(void)
{
	if(Nv.waitCnt != 0)
	{
		Nv.waitCnt--;
		return;
	}


	Nv.isLoop = TRUE;

	do
	{
		NvExecParseMsg();
		NvExecParseCmd();

	} while(Nv.isLoop == TRUE);
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseMsg(void)
{
	while(_IsSJIS(*Nv.pCur) == TRUE)
	{
		Nv.msg[Nv.msgCnt++] = *Nv.pCur++;
		Nv.msg[Nv.msgCnt++] = *Nv.pCur++;
	}
	Nv.msg[Nv.msgCnt] = '\0';


	_ASSERT(Nv.msgCnt < NV_MAX_MSG_SIZE-1);
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseCmd(void)
{
	TRACEOUT("\n[cmd:0x%x pCur:0x%x %s]\n", *Nv.pCur, Nv.pCur - Nv.pHead, FileGetScoStr(Nv.no));

	switch(*Nv.pCur++)
	{
	// !
	case 0x21:
		NvExecParseAssign();
		break;

	// $
	case 0x24:
		NvExecParseSelect();
		break;

	// %
	case 0x25:
		NvExecParsePageCall();
		break;

	// &
	case 0x26:
		NvExecParseJump();
		break;

	// <
	case 0x3c:
		NvExecParseFor();
		break;

	// > @
	case 0x3e:
	case 0x40:
		NvExecParseJumpAdr();
		break;

	// B
	case 0x42:
		NvExecParseWindow();
		break;

	// C
	case 0x43:
		NvExecParseCopy();
		break;

	// E
	case 0x45:
		NvExecParseScreenClear();
		break;

	// G
	case 0x47:
		NvExecParseImage();
		break;

	// H
	case 0x48:
		NvExecParseShowVar();
		break;

	// I
	case 0x49:
		NvExecParseKeyWait();
		break;

	// J
	case 0x4a:
		NvExecParseImageOption();
		break;

	// N
	case 0x4e:
		NvExecParseArray();
		break;

	// P
	case 0x50:
		NvExecParseScreen();
		break;

	// A, R
	case 0x41:
	case 0x52:
		NvExecParseLf();
		break;

	// Y
	case 0x59:
		NvExecParseExtensionCmd();
		break;

	// Z
	case 0x5a:
		NvExecParseSystem();
		break;

	// 
	case 0x5c:
		NvExecParseLabelCall();
		break;

	// ]
	case 0x5d:
		NvExecParseSelectStart();
		break;

	// {
	case 0x7b:
		NvExecParseIf();
		break;

	// ~
	case 0x7e:
		NvExecParseLabelCallFar();
		break;

	default:
		SystemError("[Err] NvExecParseCmd\n");
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseAssign(void)
{
	TRACEOUT("[Assign]\n");

	NvCalcVar();
	NvCalc();
	_ASSERT(Nv.calcCnt == 2);


	u16 val = NvCalcPop();
	u16 num = NvCalcPop();

	NvSetVar(num, val);
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseSelect(void)
{
	TRACEOUT("[Select]\n");

	u32 selAdr = NvCurU32();

	NvExecParseMsg();
	Nv.msgCnt = 0;
	Nv.pCur++;

	if(_Strncmp((char*)Nv.msg, "VXe", NV_MAX_SEL_BUF_SIZE) == 0)
	{
		return;
	}


	TRACEOUT("[selpush %d %s]\n", Nv.selCnt, (char*)Nv.msg);

	_Strncpy((char*)Nv.selBuf[Nv.selCnt], (char*)Nv.msg, NV_MAX_SEL_BUF_SIZE);
	Nv.selAdr[Nv.selCnt] = selAdr;
	Nv.selCnt++;
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParsePageCall(void)
{
	TRACEOUT("[PageCall]\n");

	u16 no = NvCalcGet();

	if(no == 0)
	{
		NvCallPop();
	}
	else
	{
		NvCallPushPatch(no+1, 0);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseJump(void)
{
	TRACEOUT("[Jump]\n");

	u16 no = NvCalcGet() + 1;

	// X^[g.scoiQ[I[o[j
	if(no == 1)
	{
		Nv.isLoop = FALSE;
		NvSetAct(NV_ACT_END_READY);
		return;
	}

	// G.sco
	if(no == 200)
	{
		Nv.var[150] = 3;

		NvAnime(2);
		NvSetSco(200, 0x098c);
		return;
	}

	// GfBO.sco
	if(no == 208)
	{
		NvAnime(1);
		NvSetAct(NV_ACT_END_READY);
		return;
	}

	NvSetSco(no, 0);
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseFor(void)
{
	TRACEOUT("[For]\n");

	// [v𔲂iȂj
	Nv.pCur += 3;
	Nv.pCur = Nv.pHead + NvCurU32();
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseJumpAdr(void)
{
	TRACEOUT("[JumpAdr]\n");

	Nv.pCur = Nv.pHead + NvCurU32();
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseWindow(void)
{
	u32 i;

	switch(*Nv.pCur++)
	{
	case 4:
		TRACEOUT("[window set]\n");

		for(i=0; i<6; i++)
		{
			NvCalcGet();
		}
		return;
	}

	SystemError("[Err] NvExecParseWindow");
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseCopy(void)
{
	u32 i;

	switch(*Nv.pCur++)
	{
	// E
	case 0x45:
		TRACEOUT("[Effect Copy]\n");

		for(i=0; i<9; i++)
		{
			NvCalcGet();
		}
		return;
	}

	SystemError("[Err] NvExecParseCopy");
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseScreenClear(void)
{
	switch(*Nv.pCur++)
	{
	// C
	case 0x43:
		TRACEOUT("[ScreenClear]\n");

		NvCalcGet();
		ImgSetBg(0);
		NvEffect(IMG_EFFECT_NORMAL, 0);
		return;
	}

	SystemError("[Err] NvExecParseScreenClear");
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseImage(void)
{
	switch(*Nv.pCur++)
	{
	// 0
	case 0:
		TRACEOUT("[ImageSet1]\n");

		u16 num = NvCalcGet();
		ImgSetSelect(num);
		return;

	// 1
	case 1:
		TRACEOUT("[ImageSet2]\n");

		u16 num2 = NvCalcGet();
		NvCalcGet();
		ImgSetSelect(num2);
		return;

	// S
	case 0x53:
		TRACEOUT("[ImageGetXY+Size]\n");

		NvCalcGet();
		NvCalcGet();
		return;
	}

	SystemError("[Err] NvExecParseImage");
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseShowVar(void)
{
	switch(*Nv.pCur++)
	{
	case 0x00: ;
		u16 num = NvCalcGet();

		char buf[16];
		_Sprintf(buf, "%d\n", num);

		u32 len = _Strlen(buf)-1;
		u32 i;

		for(i=0; i<len; i++)
		{
			Nv.msg[Nv.msgCnt++] = 0x82;
			Nv.msg[Nv.msgCnt++] = 0x4f + (buf[i] - 0x30);
		}
		Nv.msg[Nv.msgCnt] = '\0';


		_ASSERT(Nv.msgCnt < NV_MAX_MSG_SIZE-1);
		return;
	}

	SystemError("[Err] NvExecParseShowVar");
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseKeyWait(void)
{
	// 'K' + 1oCg΂܂
	Nv.pCur += 2;

	NvMsg();
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseImageOption(void)
{
	switch(*Nv.pCur++)
	{
	// 0
	case 0:
		TRACEOUT("[Image setXY(G cmd)]\n");

		NvCalcGet();
		NvCalcGet();
		return;
	}

	SystemError("[Err] NvExecParseImageOption");
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseArray(void)
{
	switch(*Nv.pCur++)
	{
	// C
	case 0x43: ;
		NvCalcVar();
		NvCalc();
		u16 s = NvCalcPop();
		u16 d = NvCalcGet();
		u16 i;

		TRACEOUT("[ArrayInit %d-%d]\n", s, d);

		for(i=s; i<s+d; i++)
		{
			Nv.var[i] = 0;
		}
		return;
	}

	SystemError("[Err] NvExecParseArray");
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseScreen(void)
{
	switch(*Nv.pCur++)
	{
	// F
	case 0x46:
		NvExecParseScreenFadeBlack();
		return;

	// W
	case 0x57:
		NvExecParseScreenFadeWhite();
		return;
	}

	SystemError("[Err] NvExecParseScreen");
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseScreenFadeBlack(void)
{
	u16 num  = *Nv.pCur++;
	u16 wait = NvCalcGet();

	switch(num)
	{
	case 0:
		TRACEOUT("[FadeBlack Out]\n");
		NvEffect(IMG_EFFECT_BLACK_OUT, wait);
		return;

	case 1:
		TRACEOUT("[FadeBlack In]\n");
		NvEffect(IMG_EFFECT_BLACK_IN, wait);
		return;
	}

	SystemError("[Err] NvExecParseScreenFadeBlack");
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseScreenFadeWhite(void)
{
	u16 num  = *Nv.pCur++;
	u16 wait = NvCalcGet();

	switch(num)
	{
	case 0:
		TRACEOUT("[FadeWhite Out]\n");
		NvEffect(IMG_EFFECT_WHITE_OUT, wait);
		return;

	case 1:
		TRACEOUT("[FadeWhite In]\n");
		NvEffect(IMG_EFFECT_WHITE_IN, wait);
		return;
	}

	SystemError("[Err] NvExecParseScreenFadeWhite");
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseLf(void)
{
	TRACEOUT("[Text Lf]\n");
	TextSetLf();
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseExtensionCmd(void)
{
	u16 no = NvCalcGet();
	u16 num = NvCalcGet();

	switch(no)
	{
	case 1:
		TRACEOUT("[MessageWindow Init]\n");
		// EMPTY
		return;

	case 2:
		TRACEOUT("[SysVar Init]\n");
		// EMPTY
		return;

	case 3:
		TRACEOUT("[Wait %d]\n", num);

		// GfBOiGŏI.scoj̍Ō̓m[EFCg
		if(Nv.no == 207 && Nv.waitCnt == 40)
		{
			return;
		}

		Nv.waitCnt = num;
		Nv.isLoop = FALSE;
		return;

	case 4:
		TRACEOUT("[Rnd]\n");
		Nv.var[0] = MtRand(num);
		return;
	}

	SystemError("[Err] NvExecParseY");
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseSystem(void)
{
	switch(*Nv.pCur++)
	{
	// C
	case 0x43:
		TRACEOUT("[SysCmd Change]\n");

		NvCalcGet();
		NvCalcGet();
		return;
	}

	SystemError("[Err] NvExecParseSystem");
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseLabelCall(void)
{
	TRACEOUT("[LabelCall]\n");

	u32 adr = NvCurU32();

	if(adr == 0)
	{
		NvCallPop();
	}
	else
	{
		NvCallPushPatch(Nv.no, adr);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseSelectStart(void)
{
	TRACEOUT("[SelectStart]\n");

	NvSetAct(NV_ACT_SELECT);
	Nv.isLoop = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseIf(void)
{
	TRACEOUT("[If]\n");

	u16 is  = NvCalcGet();
	u32 adr = NvCurU32();

	if(is == 0)
	{
		Nv.pCur = Nv.pHead + adr;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecParseLabelCallFar(void)
{
	TRACEOUT("[LabelCallFar]\n");

	u16 no = NvCurU16();

	if(no == 0x0000)
	{
		NvCallPop();
	}
	else if(no == 0xffff)
	{
		// ֐̖߂lϐɎ擾iȂj
		NvCalcGet();
	}
	else
	{
		u32 adr = NvCurU32();
		NvCallPushPatch(no, adr);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvCalc(void)
{
	u16 in, ed;

	for(;;)
	{
		if(*Nv.pCur == 0x7f)
		{
			Nv.pCur++;
			return;
		}

//		TRACEOUT("[calc 0x%x ptr:0x%x]\n", *Nv.pCur, Nv.pCur - Nv.pHead);

		// {Zq
		if(*Nv.pCur >= 0x74 && *Nv.pCur <= 0x7e)
		{
			in = NvCalcPop();
			ed = NvCalcPop();

			switch(*Nv.pCur)
			{
			case 0x74: NvCalcPush(ed & in);     break;
			case 0x75: NvCalcPush(ed | in);     break;
			case 0x76: NvCalcPush(ed ^ in);     break;
			case 0x77: NvCalcPush(ed * in);     break;
			case 0x78: NvCalcPush(Div(ed, in)); break;
			case 0x79: NvCalcPush(ed + in);     break;
			case 0x7a: ;
				s16 t = (s16)ed - (s16)in;
				if(t < 0)
				{
					t = 0;
				}
				NvCalcPush(t);
				break;

			case 0x7b:
				if(ed == in) NvCalcPush(1);
				else         NvCalcPush(0);
				break;

			case 0x7c:
				if(ed < in)  NvCalcPush(1);
				else         NvCalcPush(0);
				break;

			case 0x7d:
				if(ed > in)  NvCalcPush(1);
				else         NvCalcPush(0);
				break;

			case 0x7e:
				if(ed != in) NvCalcPush(1);
				else         NvCalcPush(0);
				break;
			}

			Nv.pCur++;
			continue;
		}

		// ϐ(1Byte)
		if(*Nv.pCur >= 0x80 && *Nv.pCur <= 0xbf)
		{
			u16 num = NvGetVar(*Nv.pCur - 0x80);
			NvCalcPush(num);

			Nv.pCur++;
			continue;
		}

		// ϐ(2Byte)
		if(*Nv.pCur == 0xc0 && *(Nv.pCur + 1) >= 0x40)
		{
			u16 num = NvGetVar(*(Nv.pCur + 1));
			NvCalcPush(num);

			Nv.pCur += 2;
			continue;
		}

		// 萔(1Byte)
		if(*Nv.pCur >= 0x40 && *Nv.pCur <= 0x73)
		{
			NvCalcPush(*Nv.pCur - 0x40);

			Nv.pCur++;
			continue;
		}

		// 萔(2Byte)
		if(*Nv.pCur >= 0x01 && *Nv.pCur <= 0x3f)
		{
			NvCalcPush((*Nv.pCur << 8) | *(Nv.pCur + 1));

			Nv.pCur += 2;
			continue;
		}

		// 2oCgR[h
		if(*Nv.pCur == 0x00 && *(Nv.pCur + 1) >= 0x34)
		{
			NvCalcPush((*Nv.pCur << 8) | *(Nv.pCur + 1));

			Nv.pCur += 2;
			continue;
		}
		

		SystemError("[Err] NvCalc");

	} // for
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvCalcVar(void)
{
	TRACEOUT("[var val:0x%x ptr:0x%x]\n", *Nv.pCur, Nv.pCur - Nv.pHead);

	// ϐ(1Byte)
	if(*Nv.pCur >= 0x80 && *Nv.pCur <= 0xbf)
	{
		NvCalcPush(*Nv.pCur - 0x80);

		Nv.pCur++;
		return;
	}

	// ϐ(2Byte)
	if(*Nv.pCur == 0xc0 && *(Nv.pCur + 1) >= 0x40)
	{
		NvCalcPush(*(Nv.pCur + 1));

		Nv.pCur += 2;
		return;
	}

	// 萔(1Byte)
	if(*Nv.pCur >= 0x40 && *Nv.pCur <= 0x73)
	{
		NvCalcPush(*Nv.pCur - 0x40);

		Nv.pCur++;
		return;
	}

	// 萔(2Byte)
	if(*Nv.pCur >= 0x01 && *Nv.pCur <= 0x3f)
	{
		NvCalcPush((*Nv.pCur << 8) | *(Nv.pCur + 1));

		Nv.pCur += 2;
		return;
	}

	// 2oCgR[h
	if(*Nv.pCur == 0x00 && *(Nv.pCur + 1) >= 0x34)
	{
		NvCalcPush((*Nv.pCur << 8) | *(Nv.pCur + 1));

		Nv.pCur += 2;
		return;
	}

	SystemError("[Err] NvCalcVar");
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvCalcPush(u16 num)
{
	_ASSERT(Nv.calcCnt+1 < NV_MAX_CALC_CNT);
//	TRACEOUT("[calc push: 0x%x]\n", num);

	Nv.calcTbl[Nv.calcCnt++] = num;
}
//---------------------------------------------------------------------------
EWRAM_CODE u16 NvCalcPop(void)
{
	_ASSERT(Nv.calcCnt != 0);
//	TRACEOUT("[calc pop: 0x%x]\n", Nv.calcTbl[Nv.calcCnt-1]);

	return Nv.calcTbl[--Nv.calcCnt];
}
//---------------------------------------------------------------------------
EWRAM_CODE u16 NvCalcGet(void)
{
	NvCalc();
	_ASSERT(Nv.calcCnt == 1);

	u16 ret = NvCalcPop();
	TRACEOUT("[calcGet ret:0x%x]\n", ret);

	return ret;
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvCallPush(u16 no, u32 adr)
{
	_ASSERT(Nv.callCnt < NV_MAX_CALL_CNT);
	TRACEOUT("[callpush cnt:0x%x no:0x%x adr:0x%x]\n", Nv.callCnt, no, adr);

	Nv.callNo[Nv.callCnt] = Nv.no;
	Nv.callAdr[Nv.callCnt] = Nv.pCur - Nv.pHead;
	Nv.callCnt++;

	NvSetSco(no, adr);
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvCallPushPatch(u16 no, u32 adr)
{
	// CGo^.sco
	if(no == 8)
	{
		// GfBOIÕbZ[W\
		if(Nv.var[66] == 708)
		{
			NvMsg();
		}

		return;
	}

	// MES_SUB.sco ȊO
	if(no != 4)
	{
		NvCallPush(no, adr);
		return;
	}


	TRACEOUT("[Patch no:0x%x adr:0x%x %s]\n", no ,adr, FileGetScoStr(no));

	switch(adr)
	{
	// 
	case 0x0020:
		if(Nv.var[48] != 0)
		{
			ImgSetFace(Nv.var[48]);
		}
		else
		{
			ImgSetEffectOnly(IMG_EFFECT_NORMAL, 0);
		}
		Nv.isLoop = FALSE;
		break;

	// wi{LN^{GtFNgQR
	case 0x00d5:
		ImgSetBg(Nv.var[23]);
		ImgSetChr(Nv.var[24]);
		NvEffect(IMG_EFFECT_23, 0);
		break;

	// wi{LN^{GtFNg
	case 0x0131:
		ImgSetBg(Nv.var[23]);
		ImgSetChr(Nv.var[24]);
		NvEffect(Nv.var[48], Nv.var[49]/8);
		break;

	// wi{GtFNgQR
	case 0x01f7:
		ImgSetBg(Nv.var[25]);
		NvEffect(IMG_EFFECT_23, 0);
		break;

	// wi{GtFNg
	case 0x029a:
		ImgSetBg(Nv.var[25]);
		NvEffect(Nv.var[48], Nv.var[49]/8);
		break;

	// 240x320 GtFNg
	case 0x0366:
		ImgSetBg(Nv.var[48]);
		NvEffectNoSkip(IMG_EFFECT_SCROLL_DU, 4);
		break;

	// 240x320 とGtFNg
	// V́iV@ҁjobgGhpXN[
	case 0x03c4:
		ImgSetBg(Nv.var[48]);
		NvEffectNoSkip(IMG_EFFECT_SCROLL_UD, 4);
		break;

	// ʉ
	case 0x0448:
		SePlay(Nv.var[48], FALSE);
		break;

	// ʉi[vj
	case 0x0451:
		SePlay(Nv.var[48], TRUE);
		break;

	// ʉ~߂
	case 0x045a:
		SeStop();
		break;

	// wi{ʏRs[
	// ŔiijAYCgϐgV[
	case 0x04db: ;
		u32 offset = Nv.pCur - Nv.pHead;

		if(Nv.no == 106 && (offset == 0x1235 || offset == 0x1241 || offset == 0x124d))
		{
			Nv.waitCnt = 60;
		}
		// no break

	// wi{ʏRs[
	case 0x0554:
		ImgSetBg(Nv.var[48]);
		NvEffect(IMG_EFFECT_NORMAL, 0);
		break;

	// bZ[W{L[EFCg
	case 0x059b:
		NvMsg();
		break;

	// IIH
	case 0x07d0:
		// EMPTY
		break;

	// IJnH
	case 0x077a:
		// EMPTY
		break;

	// s
	case 0x0788:
		// EMPTY
		break;

	// s
	case 0x07ac:
		// EMPTY
		break;

	// y
	case 0x0800:
		TRACEOUT("[Bgm no:0x%x]\n", Nv.var[48]+1);
		BgmPlay(Nv.var[48]+1, TRUE);
		break;

	// wi{LN^{GtFNgQR
	case 0x0835:
		ImgSetBg(Nv.var[23]);
		ImgSetChr(Nv.var[24]);
		NvEffect(IMG_EFFECT_23, 0);
		break;

	// GtFNg
	case 0x0892:
		NvAnime(3);
		break;

	// ŒXN[
	// V́iV@ҁjGfBO@AYeB
	case 0x08cb:
		ImgSetBg(688);
		NvEffectNoSkip(IMG_EFFECT_SCROLL_D, 0);
		break;

	// 240x320 GtFNg
	// V́iV@ҁjGfBO@AYeBXN[
	case 0x096a:
		ImgSetBg(688);
		NvEffectNoSkip(IMG_EFFECT_SCROLL_DU, 4);
		break;

	// ŔiijW`G̐퓬
	case 0x09bf: ;
		u32 chr = 0;

		switch(Nv.var[48])
		{
		case 0: chr = 363; break;
		case 1: chr = 362; break;
		case 2: chr = 361; break;
		case 3: chr = 360; break;
		case 4: chr = 359; break;
		case 5: chr = 364; break;
		}

		ImgSetChr(chr);
		NvEffect(IMG_EFFECT_16, 0);
		break;

	// V́iV@ҁjGfBO@[hfA{J𕔕\
	case 0x0aa4:
		switch(Nv.var[48])
		{
		case 1: ImgDirectBlend(691, 108,  0); break;
		case 2: ImgDirectBlend(692,  78, 24); break;
		case 3: ImgDirectBlend(693,   0, 58); break;
		}

		ImgSetEffectOnly(IMG_EFFECT_18, 0);
		Nv.isLoop = FALSE;
		break;

	// V́iV@ҁjGfBO@Ԑ
	case 0x0b38:
		// EMPTY
		break;

	default:
		SystemError("[Err] NvCallPushPatch 0x%x\n", adr);
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvCallPop(void)
{
	_ASSERT(Nv.callCnt != 0);

	Nv.callCnt--;
	u16 no  = Nv.callNo[Nv.callCnt];
	u16 adr = Nv.callAdr[Nv.callCnt];

	NvSetSco(no, adr);
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvMsg(void)
{
	TRACEOUT("[%s]\n", Nv.msg);

	TextSetDraw((u16*)Nv.msg, Nv.msgCnt / 2);
	TextShow();
	Nv.msgCnt = 0;

	NvSetAct(NV_ACT_KEY);
	Nv.isLoop = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvEffect(u32 idx, u32 wait)
{
	TextHide();
	TextClear();

	if(Nv.isSkip == TRUE)
	{
		idx = IMG_EFFECT_NORMAL;
		wait = 0;
	}
	ImgSetEffect(idx, wait);

	Nv.isLoop = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvEffectNoSkip(u32 idx, u32 wait)
{
	TextHide();
	TextClear();
	ImgSetEffect(idx, wait);

	Nv.isLoop = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvAnime(u32 no)
{
	TextHide();
	TextClear();
	AnimeSetDat(no);

	Nv.isLoop = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE u16 NvCurU16(void)
{
	u16 num = *Nv.pCur++;

	num |= ((*Nv.pCur++) << 8);

	return num;
}
//---------------------------------------------------------------------------
EWRAM_CODE u32 NvCurU32(void)
{
	u32 num = *Nv.pCur++;

	num |= ((*Nv.pCur++) << 8);
	num |= ((*Nv.pCur++) << 16);
	num |= ((*Nv.pCur++) << 24);

	return num;
}
