#include "siori.h"
#include "libmy/sram.h"

#include "bgm.h"
#include "img.h"
#include "nv.h"
#include "text.h"

#include "se.h"
#include "history.h"


//---------------------------------------------------------------------------
// OQ
extern ST_BGM  Bgm;
extern ST_IMG  Img;
extern ST_NV   Nv;
extern ST_TEXT Text;


//---------------------------------------------------------------------------
ST_SIORI Siori;


//---------------------------------------------------------------------------
EWRAM_CODE void SioriInit(void)
{
	_Memset(&Siori, 0x00, sizeof(ST_SIORI));

	Siori.size = 2 + TEXT_SIORI_CX*2 + sizeof(ST_BGM) + sizeof(ST_IMG) + sizeof(ST_NV) + sizeof(ST_TEXT);

	TRACEOUT("[SioriSize: 0x%x]\n", Siori.size);
	_ASSERT(Siori.size * SIORI_MAX_CNT < SIORI_SRAM_SIZE);
}
//---------------------------------------------------------------------------
EWRAM_CODE void SioriSave(u32 no)
{
	TRACEOUT("[Siori Save no:%d]\n", no);
	_ASSERT(no < SIORI_MAX_CNT);

	u32 i;
	u8* p;

	SramSeek(Siori.size * no);
	SramWrite8('D');
	SramWrite8('I');

	// ^Cg
	p = (u8*)TextGetBufSiori();
	for(i=0; i<TEXT_SIORI_CX*2; i++)
	{
		SramWrite8(*p++);
	}

	// BGM
	p = (u8*)&Bgm;
	for(i=0; i<sizeof(ST_BGM); i++)
	{
		SramWrite8(*p++);
	}

	// Img
	p = (u8*)&Img;
	for(i=0; i<sizeof(ST_IMG); i++)
	{
		SramWrite8(*p++);
	}

	// Nv
	p = (u8*)&Nv;
	for(i=0; i<sizeof(ST_NV); i++)
	{
		SramWrite8(*p++);
	}

	// Text
	p = (u8*)&Text;
	for(i=0; i<sizeof(ST_TEXT); i++)
	{
		SramWrite8(*p++);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE bool SioriLoad(u32 no)
{
	TRACEOUT("[Siori Load no:%d]\n", no);

	if(SioriIsSram(no) == FALSE)
	{
		TRACEOUT("[Failure]\n");
		return FALSE;
	}


	u32 i;
	u8* p;

	SramSeek(Siori.size * no + 2 + TEXT_SIORI_CX*2);

	// BGM
	p = (u8*)&Bgm;
	for(i=0; i<sizeof(ST_BGM); i++)
	{
		*p++ = SramRead8();
	}

	// Img
	p = (u8*)&Img;
	for(i=0; i<sizeof(ST_IMG); i++)
	{
		*p++ = SramRead8();
	}

	// Nv
	p = (u8*)&Nv;
	for(i=0; i<sizeof(ST_NV); i++)
	{
		*p++ = SramRead8();
	}

	// Text
	p = (u8*)&Text;
	for(i=0; i<sizeof(ST_TEXT); i++)
	{
		*p++ = SramRead8();
	}

	// A
	SioriRestart();

	return TRUE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void SioriRestart(void)
{
	HistoryInit();
	BgmRestart();
	SeStop();

	TextShow();
	ImgSetFade(Img.fadeCnt);

	// Ȉꍇ
	if(Nv.act == NV_ACT_SELECT)
	{
		Nv.actStep = 0;
	}

	// V̔sk@AYeBiGV.scoj
	if(Img.bgNo == 666)
	{
		ImgSetEffect(IMG_EFFECT_SCROLL_D, 0);
		return;
	}

	// GfBÕfA{JiGŏI.scoj
	if(Img.bgNo == 687)
	{
		ImgDirectCrop(687, 0, 0);

		u32 offset = Nv.pCur - Nv.pHead;

		if(offset >= 0x2aaa)
		{
			ImgDirectBlend(691, 108, 0);
		}

		if(offset >= 0x2c38)
		{
			ImgDirectBlend(692, 78, 24);
		}

		if(offset >= 0x2dfd)
		{
			ImgDirectBlend(693, 0, 58);
		}

		ImgSetEffectOnly(IMG_EFFECT_NORMAL, 0);
		return;
	}

	ImgSetEffect(IMG_EFFECT_NORMAL, 0);
}
//---------------------------------------------------------------------------
EWRAM_CODE char* SioriGetName(u32 no)
{
	if(SioriIsSram(no) == FALSE)
	{
		return "|||||||||||";
	}

	SramSeek(Siori.size * no + 2);
	return (char*)SramGetPointer();
}
//---------------------------------------------------------------------------
EWRAM_CODE bool SioriIsSram(u32 no)
{
	SramSeek(Siori.size * no);

	if(SramRead8() != 'D') return FALSE;
	if(SramRead8() != 'I') return FALSE;

	return TRUE;
}
