/*
 * pms256.c  PMS256 Ÿ
 *
 * Copyright (C) 1997-1998 Masaki Chikama (Wren) <chikama@kasumi.ipl.mech.nagoya-u.ac.jp>
 *               1998-                           <masaki-c@is.aist-nara.ac.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *          0.00    97/11/06 
 *          0.01    97/11/13 إå֤ѹ
 *          0.01-01 97/11/27 Ҥѹ
 * @version 0.02    98/02/26 ǥХåλ
 *
*/
/* $Id: pms256.c,v 1.1.1.1 1999/04/17 13:14:15 chikama Exp $ */

#include <string.h>
#include "portab.h"
#include "LittleEndian.h"
#include "cg.h"

static Pms_header pms256;
static Pallet256 pal;

static void
pms256_getPal(const BYTE *b) {
	int i;
	int ptr = pms256.pmsPp;
	for (i = 0; i < 256; i ++) {
		pal.red[i]   = b[ptr++];
		pal.green[i] = b[ptr++];
		pal.blue[i]  = b[ptr++];
	}
}

extern void
pms256_getHdr(const BYTE *data, Pms_header *pms) {
	pms->pmsVer = LittleEndian_getW(data, 2);
	pms->pmsHdrSize = LittleEndian_getW(data, 4);
	pms->pmsBpp = data[6];
	pms->pmsBppS = data[7];
	pms->pmsSf = data[8];
	pms->pmsBf = LittleEndian_getW(data, 10);
	pms->pmsX0 = LittleEndian_getDW(data, 16);
	pms->pmsY0 = LittleEndian_getDW(data, 20);
	pms->pmsXW = LittleEndian_getDW(data, 24);
	pms->pmsYW = LittleEndian_getDW(data, 28);
	pms->pmsDp = LittleEndian_getDW(data, 32);
	pms->pmsPp = LittleEndian_getDW(data, 36);
	pms->pmsCp = LittleEndian_getDW(data, 40);
}

extern void
pms256_expand_8bpp(const BYTE *b, BYTE *pic, int ptr) {
	int c0, c1;
	int x, y, loc, l, i;
	int scanline = pms256.pmsXW;
	
	b += ptr;
	for (y = 0; y < pms256.pmsYW; y ++) {
		for (x = 0; x < pms256.pmsXW; ) {
			loc = y * scanline + x;
			c0 = *b++;
			if (c0 <= 0xf7) {
				*(pic + loc) = c0; x++;
			} else if (c0 == 0xff) {
				l = (*b) + 3; x+=l; b++;
				memcpy (pic + loc, pic + loc - scanline, l);
			} else if (c0 == 0xfe) {
				l = (*b) + 3; x+=l; b++;
				memcpy (pic + loc, pic + loc - scanline * 2, l);
			} else if (c0 == 0xfd) {
				l = (*b) + 4; x+=l; b++;
				c0 = *b++;
				memset(pic + loc, c0, l);
			} else if (c0 == 0xfc) {
				l = ((*b) + 3)  * 2; x+=l; b++;
				c0 = *b++; c1 = *b++;
				for (i = 0; i < l; i+=2) {
					*(pic + loc + i    ) = c0;
					*(pic + loc + i + 1) = c1;
				}
			} else {
				*(pic + loc) = *b++; x++;
			}
		}
	}
}

extern boolean
Pms256_checkFmt(const BYTE *data) {
	if (data[0] != 0x50 || data[1] != 0x4d)   { return false; }
	pms256_getHdr(data, &pms256);
	if (pms256.pmsBpp != 8)                   { return false; }
	if (pms256.pmsX0 < 0 || pms256.pmsY0 < 0) { return false; }
	pms256_getPal(data);
	return true;
}

extern void
Pms256_expandCg(const BYTE *data, BYTE *pic) {
	pms256_expand_8bpp(data, pic, pms256.pmsDp);
}

extern Pms_header *
Pms256_getCgInfo() {
	return &pms256;
}

extern void
Pms256_getInitLoc(MyPoint *p) {
	p->x = pms256.pmsX0;
	p->y = pms256.pmsY0;
}

extern void
Pms256_getSize(MyDimension *d) {
	d->width  = pms256.pmsXW;
	d->height = pms256.pmsYW;
}

extern Pallet256 *
Pms256_getPallet() {
	return &pal;
}
