#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "portab.h"
#include "cg.h"
#include "LittleEndian.h"

#define CG_MAX_X 800
#define CG_MAX_Y 960

static WORD picbufw[CG_MAX_X*CG_MAX_Y];
static BYTE picbufb[CG_MAX_X*CG_MAX_Y];

static Pms_header *pms_h;
static Pallet256 *pal;

static char *filein;
static char *fileout;
static char *filealpha;

/*
	PMS2 (new PMS + new link) -> BMP converter for UNIX
	
	Ver 0.01 : 1999-04-17 
	Ver 0.02 : 2010-01-07 ե֥ɤΥإå
*/

void usage(void)
{
	puts("Usage : pms2bmp {-h|-v} filein fileout {alpha}");
	puts("     -h: show this help.");
	puts("     -v: show version");
	puts(" filein: input pmsfile");
	puts("fileout: output bmpfile");
	puts("  alpha: output alphaimage bmpfile (if exist)"); 
	exit(2);
}

void copyright(void)
{
	puts("ALICE SOFT PMS2 -> BMP Convert Utility Ver 0.1 (C)Wren(Masaki Chikama)");
}

void err_msg(int msg_no)
{
	static char *errormsg[]={
		"pms2bmp : ꡼­ޤ",           /* 0 */
		"pms2bmp : PMSեǤϤޤ",      /* 1 */
		"pms2bmp : ¾Υ顼"
	};
	puts(errormsg[msg_no]);
	exit(1);
}

/* save alpha image */
void savealpha(FILE *fp)
{
	int i, y;
	BYTE bmp_pal[256*4];
	BYTE *b;

	BYTE bmp_fileh[14];
	BYTE bmp_infoh[40];
	
	memset(bmp_fileh, 0, sizeof(bmp_fileh));
	memset(bmp_infoh, 0, sizeof(bmp_infoh));

	bmp_fileh[0] = 'B';
	bmp_fileh[1] = 'M';
	LittleEndian_putDW(sizeof(bmp_fileh) + sizeof(bmp_infoh) +
			   256*4 +
			   pms_h->pmsXW * pms_h->pmsYW * 1,
			   bmp_fileh, 2);
	LittleEndian_putDW(sizeof(bmp_fileh) + sizeof(bmp_infoh) +
			  256*4,
			  bmp_fileh, 10);

	LittleEndian_putDW(sizeof(bmp_infoh), bmp_infoh, 0);
	LittleEndian_putDW(pms_h->pmsXW, bmp_infoh, 4);
	LittleEndian_putDW(pms_h->pmsYW, bmp_infoh, 8);
	LittleEndian_putW(1, bmp_infoh, 12);
	LittleEndian_putW(8, bmp_infoh, 14);
	
	for (i=0;i<256;i++) {
		bmp_pal[i*4+0] = i;
		bmp_pal[i*4+1] = i;
		bmp_pal[i*4+2] = i;
		bmp_pal[i*4+3] = 0;
	}
	
	fwrite(&bmp_fileh, sizeof(bmp_fileh),1,fp);
	fwrite(&bmp_infoh, sizeof(bmp_infoh),1,fp);
	
	fwrite(&bmp_pal,256*4,1,fp);
	
	for (y=(WORD)pms_h->pmsYW-1;y>=0;y--) {
		b = &picbufb[y*pms_h->pmsXW];
		fwrite(b,pms_h->pmsXW,1,fp);
		if (pms_h->pmsXW%4 != 0) {
			for (i=0;i<4-pms_h->pmsXW%4;i++){
				fputc(0,fp);
			}
		}
	}
}

/* save bmp image (8/16) */
void savebmp(FILE *fp)
{
	int i,x,y;
	WORD w0;
	BYTE bmp_pal[256*4];
	BYTE *b,*b1;

	BYTE bmp_fileh[14];
	BYTE bmp_infoh[40];
	
	memset(bmp_fileh, 0, sizeof(bmp_fileh));
	memset(bmp_infoh, 0, sizeof(bmp_infoh));

	if (NULL == (b1 = malloc(CG_MAX_X*3))) err_msg(0);
	
	bmp_fileh[0] = 'B';
	bmp_fileh[1] = 'M';
	LittleEndian_putDW(sizeof(bmp_fileh) + sizeof(bmp_infoh) +
			   ((pms_h->pmsBpp==8)?256*4:0) +
			   pms_h->pmsXW * pms_h->pmsYW * (pms_h->pmsBpp==8?1:3),
			   bmp_fileh, 2);
	LittleEndian_putDW(sizeof(bmp_fileh) + sizeof(bmp_infoh) +
			   (pms_h->pmsBpp==8?256*4:0),
			   bmp_fileh, 10);

	LittleEndian_putDW(sizeof(bmp_infoh),    bmp_infoh, 0);
	LittleEndian_putDW(pms_h->pmsXW,         bmp_infoh, 4);
	LittleEndian_putDW(pms_h->pmsYW,         bmp_infoh, 8);
	LittleEndian_putW(1,                     bmp_infoh, 12);
	LittleEndian_putW(pms_h->pmsBpp==8?8:24, bmp_infoh, 14);
	
	if (pms_h->pmsBpp == 8) {
		for (i=0;i<256;i++) {
			bmp_pal[i*4+0] = pal->blue[i];
			bmp_pal[i*4+1] = pal->green[i];
			bmp_pal[i*4+2] = pal->red[i];
			bmp_pal[i*4+3] = 0;
		}
	}
	
	fwrite(&bmp_fileh, sizeof(bmp_fileh),1,fp);
	fwrite(&bmp_infoh, sizeof(bmp_infoh),1,fp);
	
	if (pms_h->pmsBpp==8) {
		fwrite(&bmp_pal,256*4,1,fp);
	}
	
	for (y=(WORD)pms_h->pmsYW-1;y>=0;y--) {
		b = pms_h->pmsBpp==8 ? &picbufb[y*pms_h->pmsXW] : (BYTE *)&picbufw[y*pms_h->pmsXW];
		if (pms_h->pmsBpp==8) {
			fwrite(b,pms_h->pmsXW,1,fp);
			if (pms_h->pmsXW%4 != 0) {
				for (i=0;i<4-pms_h->pmsXW%4;i++){
					fputc(0,fp);
				}
			}
		} else {
			for (x=0;x<pms_h->pmsXW;x++) {
				w0=b[x*2+0]+b[x*2+1]*256;
				b1[x*3+2]=((w0>>11)<<3)|0x7;
				b1[x*3+1]=(((w0>>5)&0x3f)<<2)|0x3;
				b1[x*3+0]=((w0&0x1f)<<3)|0x7;
			}
			fwrite(b1,pms_h->pmsXW*3,1,fp);
			if ((pms_h->pmsXW*3)%4 !=0) {
				for (i=0;i<4-(pms_h->pmsXW*3)%4;i++){
					fputc(0,fp);
				}
			}
		}
	}
	free(b1);
}

/* check pms cg format */
static CG_TYPE check_cgformat(const BYTE *data)
{
        if (Pms256_checkFmt(data)) {
                return ALCG_PMS8;
        } else if (Pms64k_checkFmt(data)) {
                return ALCG_PMS16;
        }
        return ALCG_UNKNOWN;
}

/* convert PMS to BMP */
static void pms2bmp()
{
	FILE *in, *out;
	int i, size;
	char *b;
	boolean hasAlpha;

	if (NULL == (in = fopen(filein, "rb"))) {
		fprintf(stderr, "file (%s) can't open\n", filein);
		exit(1);
	}
	if (NULL == (out = fopen(fileout, "wb"))) {
		fprintf(stderr, "file (%s) can't open\n", fileout);
		exit(1);
	}
	
	fseek(in, 0, SEEK_END);
	size = ftell(in);
	fseek(in, 0, SEEK_SET);
	
	if (NULL == (b = malloc(size +1))) err_msg(0);
	fread(b, 1, size+1, in);
	fclose(in);
	
	switch(check_cgformat(b)) {
	case ALCG_PMS8:
		Pms256_expandCg(b, picbufb);
		pms_h = Pms256_getCgInfo();
		pal = Pms256_getPallet();
		break;
	case ALCG_PMS16:
		Pms64k_expandCg(b, picbufw, &hasAlpha, picbufb);
		pms_h = Pms64k_getCgInfo();
		break;
	case ALCG_UNKNOWN:
	case ALCG_VSP:
	case ALCG_BMP8:
	case ALCG_BMP24:
	default:
		err_msg(1); break;
	}

	savebmp(out);
	fclose(out);

	if (hasAlpha && filealpha != NULL) {
		if (NULL == (out = fopen(filealpha, "wb"))) {
			fprintf(stderr, "file (%s) can't open\n", filealpha);
			exit(1);
		}
		savealpha(out);
		fclose(out);
	}
	
	printf("convert ... %s ", fileout);
	
	/* show pms status */
	printf("0000,0000,%04d,%04d,",pms_h->pmsXW,pms_h->pmsYW);
	for (i = 0; i < 16; i++) {
		putchar('0' + ((pms_h->pmsBf & (1 << i))? 1 : 0));
	}
	printf(",%03d,", pms_h->pmsSf);
	printf("%04d,%04d\n",pms_h->pmsX0, pms_h->pmsY0);
}

static void opt_check(int argc, char **argv)
{
	if (**argv=='-') {
		(*argv)++;
		if (**argv == '?' || **argv == 'h') {
			usage();
		} else if (**argv=='v') {
			copyright();
			exit(0);
		} else {
			usage();
		}
	}
	
	if (argc >= 2) {
		if (NULL == (filein = malloc(strlen(*argv) +1))) err_msg(0);
		strcpy(filein, *argv);
		argv++;
	}
	if (argc >= 3) {
		if (NULL == (fileout = malloc(strlen(*argv) +1))) err_msg(0);
		strcpy(fileout, *argv);
		argv++;
	}
	if (argc >= 4) {
		if (NULL == (filealpha = malloc(strlen(*argv) +1))) err_msg(0);
		strcpy(filealpha, *argv);
		argv++;
	}
}

int main(int argc,char **argv)
{
	if (argc >= 2) {
		opt_check(argc,++argv);
	} else {
		usage();
	}
	pms2bmp();
	return 0;
}
